"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AnalyticsManager = void 0;

var _domTracker = require("./dom-tracker");

var _performanceTracker = require("./performance-tracker");

var _types = require("./types");

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * The `analytics-manager` module contains the `AnalyticsManager` class, the
 * main entry point to Mobify's analytics framework. You can use it to send
 * analytics data to several analytics connectors at the same time.
 * @module progressive-web-sdk/dist/analytics-integrations/analytics-manager
 */
const isClientSide = typeof window !== 'undefined';
/* istanbul ignore next */

const ENV = process.env.NODE_ENV || 'development';
/**
 * This is a composite class. When you call `track('event', {payload})` on
 * the AnalyticsManager, it will call the `track()` method on every configured
 * Connector, in turn.
 *
 * @implements {module:progressive-web-sdk/dist/analytics-integrations/interface.AnalyticsConnector}
 */

class AnalyticsManager {
  /**
   * @constructor
   *
   * @param {Object} options Analytics Manager options
   * @param {Array<module:progressive-web-sdk/dist/analytics-integrations/interface.AnalyticsConnector>} options.connectors
   *     An array of Connectors to register with the Analytics Manager. These must
   *     implement the AnalyticsConnector interface.
   * @param {Boolean} options.debug debug mode flag. `true` will turn the logger on.
   */
  constructor({
    connectors = [],
    debug = false
  } = {}) {
    this.connectors = connectors;
    this.debug = debug;
    this.queue = [];
    this.loaded = false;
    this.performanceTracker = new _performanceTracker.PerformanceTracker(this);
    /* istanbul ignore next */

    if (isClientSide) {
      this.domTracker = new _domTracker.DOMTracker(this);
      this.load().catch(e => console.error(e));
    }
  }
  /**
   * @inheritDoc
   */


  load() {
    return Promise.all(this.connectors.map(connector => connector.load())).then(() => {
      this.loaded = true;
      this.drainQueue();
    }).catch(() => {
      return Promise.reject('Error loading analytics');
    });
  }
  /**
   * Validate and transform the analytics event and send it to its analytics connectors.
   * @inheritDoc
   */


  track(type, data) {
    if (!type) {
      throw new Error('Please specify Analytics type to track.');
    }
    /* istanbul ignore else */


    if (ENV !== 'production' || this.debug) {
      switch (type) {
        case _types.PAGEVIEW:
          data = (0, _types.validate)(_types.page, data);
          break;

        case _types.OFFLINE:
          data = (0, _types.validate)(_types.offline, data);
          break;

        case _types.UIINTERACTION:
          data = (0, _types.validate)(_types.uiInteraction, data);
          break;

        case _types.PERFORMANCE:
          data = (0, _types.validate)(_types.performance, data);
          break;

        case _types.ADDTOCART:
        case _types.REMOVEFROMCART:
        case _types.ADDTOWISHLIST:
        case _types.REMOVEFROMWISHLIST:
          data = (0, _types.validate)(_types.shoppingList, data);
          break;

        case _types.PURCHASE:
          data = (0, _types.validate)(_types.purchase, data);
          break;

        case _types.PRODUCTIMPRESSION:
          data = (0, _types.validate)(_types.product, data);
          break;

        case _types.ERROR:
          data = (0, _types.validate)(_types.error, data);
          break;

        case _types.APPLEPAYOPTIONDISPLAYED:
        case _types.APPLEPAYBUTTONDISPLAYED:
        case _types.APPLEPAYBUTTONCLICKED:
          // these events do not require any further data.
          break;

        case _types.LOCALE:
          data = (0, _types.validate)(_types.locale, data);
          break;

        default:
          break;
      }
    }

    this.queue.push({
      type,
      data
    });

    if (!isClientSide || !window.navigator.onLine || !this.loaded) {
      console.info(`Event '${type}' is pushed to queue`);
      return;
    }

    this.drainQueue();
    return data;
  }
  /**
   * @private
   */


  drainQueue() {
    while (this.queue.length) {
      const {
        type,
        data
      } = this.queue.shift();
      this.connectors.forEach(connector => {
        const trackedData = connector.track(type, data);
        if (this.debug && trackedData) this.log(connector, type, trackedData);
      });
    }
  }
  /**
   * Log an analytics event to console.
   *
   * @param {String} connector The connector for an analytics provider.
   * @param {String<module:Types>} type Analytics event type.
   * @param {Object} data The data sent for tracking for the analytics event.
   *
   * @private
   */


  log(connector, type, data) {
    console.groupCollapsed('%c%s%c sends a %c%s%c event', 'color: #006EFF; font-size: 13px; line-height:30px;', connector.constructor.name, 'color: inherit', 'color: #FF566A; font-size: 13px', type, 'color: inherit');
    console.log(data);
    console.groupEnd();
  }
  /**
   * Track performance metrics for page loads.
   *
   * If the window.performance API is unavailable, this does nothing.
   *
   * @param {Object} pageLoad
   * @param {Number} pageLoad.start
   * @param {Number} pageLoad.end
   * @returns {Promise}
   */


  trackPageLoad(pageLoad) {
    return this.performanceTracker.trackPageLoad(pageLoad);
  }

}

exports.AnalyticsManager = AnalyticsManager;