"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _button = _interopRequireDefault(require("../button"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * The `LazyLoader` makes it possible to delay rendering of content until the
 * user has actually scrolled a certain distance through a page.
 *
 * A few common patterns for this component include:
 *
 * * Lazily fetch contents from a server when the user scrolls that content
 *   into view.
 * * Lazily render items, if rendering all of the items at once is a
 *   performance concern.
 *
 * The way this works is: when the `LazyLoader` scrolls into view, it triggers
 * a function callback to `fetchItems` (a Promise) to achieve the various lazy
 * loading effects. See example usages below.
 *
 * The LazyLoader is not responsible for managing its children.
 * LazyLoader children must be managed externally to the LazyLoader
 * (e.g. in the LazyLoader's parent)
 *
 * @example ./DESIGN.md
 */
class LazyLoader extends _react.default.Component {
  constructor(props) {
    super(props);
    this.state = {
      shouldNotifyUser: false,
      loading: false,
      done: false
    };
    this.handleScroll = this.handleScroll.bind(this);
    this.loadNext = this.loadNext.bind(this);
    this.reset = this.reset.bind(this);
    this.checkIfDone = this.checkIfDone.bind(this);
  }

  componentDidMount() {
    window.addEventListener('scroll', this.handleScroll);
    this.checkIfDone();
  }

  componentWillReceiveProps(nextProps) {
    this.checkIfDone(nextProps);
  }

  componentWillUnmount() {
    window.removeEventListener('scroll', this.handleScroll);
  }

  reset() {
    this.setState({
      loading: false,
      done: false
    });
  }

  loadNext() {
    const {
      currentItemCount,
      itemsPerPage,
      fetchItems
    } = this.props;
    this.setState({
      loading: true,
      shouldNotifyUser: false
    });
    fetchItems && fetchItems({
      startPosition: currentItemCount,
      lastPosition: currentItemCount + itemsPerPage
    }).then(() => {
      this.setState({
        loading: false,
        shouldNotifyUser: true
      });
    });
  }

  handleScroll() {
    const {
      useLoadMoreButton,
      scrollPixelOffset
    } = this.props;
    const {
      loading,
      done
    } = this.state;

    if (loading || done || useLoadMoreButton) {
      return;
    }

    const bottomScrollThreshold = this.el.getBoundingClientRect().bottom;
    const pixelOffset = scrollPixelOffset >= 0 ? scrollPixelOffset : 0;

    if (bottomScrollThreshold <= window.innerHeight + pixelOffset) {
      this.loadNext();
    }
  }

  checkIfDone(props) {
    const {
      currentItemCount,
      itemTotal
    } = props || this.props;
    const {
      done
    } = this.state;

    if (currentItemCount >= itemTotal && !done) {
      this.setState({
        done: true,
        loading: false
      });
    }
  }

  render() {
    const {
      allItemsLoadedMessage,
      notificationAddedMessage,
      children,
      className,
      loadMoreButtonClassName,
      loadMoreItemsMessage,
      loadingIndicator,
      relevantText,
      useLoadMoreButton
    } = this.props;
    const {
      done,
      loading,
      shouldNotifyUser
    } = this.state;
    const classes = (0, _classnames.default)('pw-lazy-loader', className);
    const loadMoreButtonClasses = (0, _classnames.default)('pw-lazy-loader__load-more', loadMoreButtonClassName);
    return /*#__PURE__*/_react.default.createElement("div", {
      className: classes,
      ref: el => {
        this.el = el;
      }
    }, children, useLoadMoreButton && /*#__PURE__*/_react.default.createElement(_button.default, {
      className: loadMoreButtonClasses,
      onClick: this.loadNext,
      disabled: loading || done
    }, done ? allItemsLoadedMessage : loadMoreItemsMessage), loading && /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-lazy-loader__indicator"
    }, loadingIndicator), /*#__PURE__*/_react.default.createElement("div", {
      className: "u-visually-hidden",
      "aria-live": "polite",
      "aria-relevant": relevantText,
      tabIndex: "-1"
    }, shouldNotifyUser && notificationAddedMessage));
  }

}

LazyLoader.defaultProps = {
  allItemsLoadedMessage: 'All items loaded',
  itemsPerPage: 20,
  itemTotal: Number.POSITIVE_INFINITY,
  loadMoreItemsMessage: 'Load more items',
  loadingIndicator: 'Loading...',
  notificationAddedMessage: 'More items loaded',
  relevantText: 'additions text',
  scrollPixelOffset: 0
};
LazyLoader.propTypes = {
  /**
   * The current number of items. We cannot determine this based on the number
   * of children in case the children are wrapped in another component, such
   * as a Grid component providing layout.
   */
  currentItemCount: _propTypes.default.node.isRequired,

  /**
   * A user-defined function that is called when the bottom of the LazyLoader
   * component scrolls into view. This provides the user with a hook to
   * perform network requests to fetch and load more content. This function
   * must return a promise that resolves when any fetched content is loaded.
   */
  fetchItems: _propTypes.default.func.isRequired,

  /**
   * A message used on the Load More Items button when all items have loaded.
   * Exposed to screen readers.
   */
  allItemsLoadedMessage: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * The children to be rendered.
   */
  children: _propTypes.default.node,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * A user-defined number that indicates the end state of the LazyLoader.
   * When the LazyLoader has loaded this many items, then it knows it can stop
   * attempting to load more items.
   */
  itemTotal: _propTypes.default.number,

  /**
   * The number of items to load for each page.
   */
  itemsPerPage: _propTypes.default.number,

  /**
   * Adds values to the `class` attribute of the load more button.
   */
  loadMoreButtonClassName: _propTypes.default.string,

  /**
   * A message used on the Load More Items button. Exposed to screen readers.
   */
  loadMoreItemsMessage: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * An indicator that the next page of contents is loading.
   */
  loadingIndicator: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * A message exposed to screen readers when new contents have been loaded.
   */
  notificationAddedMessage: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * The `LazyLoader` component uses [ARIA Live Regions](https://developer.mozilla.org/en-US/docs/Web/Accessibility/ARIA/ARIA_Live_Regions)
   * to report updates to users using assistive tools like screen readers.
   * The `relevantText` prop describes which types of changes are relevant to
   * the Live Region, which tells the assistive tool which changes to report
   * to the user. This text is fed into an `aria-relevant` prop, accepts a
   * space delimited list of one or more of the following values: `additions`,
   * `removals`, `text` or `all`. See [here](https://developer.mozilla.org/en-US/docs/Web/Accessibility/ARIA/ARIA_Techniques/Using_the_aria-relevant_attribute)
   * for details.
   */
  relevantText: _propTypes.default.string,

  /**
   * The pixel offset added to window.innerHeight to move the threshold for when
   * new content is fetched.
   *
   * New content is loaded when the bottom of the LazyLoader is scrolled into view
   * (the bottom position is less than or equal to window.innerHeight).
   * E.g. with a 100px offset new content will begin to load when LazyLoader bottom
   * is 100px below the bottom of the window.
   */
  scrollPixelOffset: _propTypes.default.number,

  /**
   * Indicates if the load more button should be used instead of scrolling.
   */
  useLoadMoreButton: _propTypes.default.bool
};
var _default = LazyLoader;
exports.default = _default;